%% IRIS State-Space -- Essentials (i)
% by Michal Andrle (IMF)
%
% This simple tutorial demonstrates how to obtain and utilize the
% state-space form of the model implied by IRIS first-order (log)-linear
% solution.
%
% A simple transformation of the IRIS state-space form to a |textbook| one
% is provided and a very simple simulation is contrasted to results from
% IRIS |simulate| command.
%
% Using the state-space form is the FIRST and ESSENTIAL step towards
% building your own computational routines, e.g. advanced use of the Kalman
% filter, etc.

%% Housekeeping
clear all; close all; clc;


%% Reading the DSGE model
% Loading the state-space model, a non-stationary one.
load('read_model.mat');


%% Get the state-space form
% To get the state-space form in IRIS, use the model property |sspace|,
% which returns all the matrices needed. See also a |help/sspace|
% |Omega| is the covariance matrix of all shocks, note that transition and
% measurement errors shocks are stacked into one vector. The proper loading
% is guaranteed by |R| and |H| matrices.
%
% Variable |list| returned from the |sspace()| command is a cell array of
% measurement variables names, transition names including auxilliary lags
% and shocks. 

% please, read Mirek's documentation 
help model/sspace

% get the sspace without the Schur transform (triangular= false)
[T R K Z H D U Omega list] = sspace(m,'triangular',false);

% check the structure of the transition vector. note lags and logs...
list{2}'

%% Transform the IRIS state-space into a |textbook| form
% Let's transform the IRIS state-space form, which is rather an efficient
% one from the computational point of view, to a standard, textbook for as
% can be found in Harvey's 1989 book on Kalman filter... It is trivial,
% once the trinagular form is corrected for, see |help @model/sspace|

% get dimensions
[ny na2] = size(Z);
[nx na1] = size(T);
[ans  ne ] = size(H);

if eq(na1,na2)
    na = na1;
else
    % one will never get here...
    error('Hmmm something is wrong...');
end

nxf = nx - na;

% nex |X| state vector is X := [XF; ALPHA]
% create a new transition matrix...
TT = zeros(nx, nx);
    TT(:,1:nxf) = 0;
    TT(:,nxf+1:end) = T; % TT = [0 T];
    
% create a new measurement matrix...
ZZ = zeros(ny, nx);
    ZZ(:,nxf+1:end) = Z; % ZZ = [0 Z];

%% Simulate a simple impulse response simulation
shock2simulate = 'Ea';
shockvalue     = 0.01;
nper           = 40.0;

% simulation
shoxNames = list{3}; % shox names = eList...

% assign the simulation (path, IRF, anything...)
ee = zeros(ne, nper);
    ix = find(strcmpi(shock2simulate,shoxNames));
    ee(ix,1) = shockvalue;

%% Simulate a path of shocks
% Due to a non-stationarity of the model, we will simulate deviations
% around its balanced growth path. We will ignore the initial conditions
% (same for both shock and control simulation) and constants/drifts. For
% this purpose a variable |dev| is created, taking the value of zero.

% allocate the simulation path
yy = zeros(ny, nper);
xx = zeros(nx, nper);

x_init = zeros(nx,1); % initial conditions...
dev    = 0;

% textbook form... sound, familiar ha? 
t = 1;
    xx(:,1) = dev*(K + x_init) + R*ee(:,1);
    yy(:,1) = ZZ*xx(:,1) + dev*D + H*ee(:,1);
for t = 2  : nper
    xx(:,t) = TT*xx(:,t-1) + dev*K + R*ee(:,t);
    yy(:,t) = ZZ*xx(:,t  ) + dev*D + H*ee(:,t);
end



%% Simulate the IRIS simulation...
% This is a standard IRIS routine for simulating a path of shocks, in this
% case leading just to a simple impulse...

dbss = sstatedb(m,1:nper);
dbss.(shock2simulate)(1) = shockvalue;
ds = simulate(m, dbss, 1:nper, 'anticipate', false,'deviation',false);
    

%% Compare with the IRIS solution... 
% In our particular case, due to the model structure, neither of
% observables is in logs, so we can directly compare the results to IRIS
% simulation... This will NOT work for other transition variables. For
% those one needs to treat logs if needed.

ylist = get(m,'ylist');
f = figure();
    for i = 1 : numel(ylist)
        subplot(2,2,i);
            tmp = ((ds.(ylist{i}))-(dbss.(ylist{i})));
            pp = plot([yy(i,:)' tmp(1:nper)]);
                set(pp(1),'linewidth',3,'color','b');
                set(pp(2),'linewidth',2,'color','r','linestyle','--');
            legend('poor man','iris');
            title(ylist{i},'interpreter','none');
            grid on;
            grfun.zeroline();
    end
    ftitle('Comparing IRIS and hand-made solution');

%% Make the units right (log or non-log?)
% The model can be log-linearized for some variables and linearized in
% terms the others. This is important to understand and treat properly.
% |islog| property of the model obj. returns a structure with 0/1
% indicating if it the variable is in logs.

islog = get(m,'islog')


%% Help for the functions used in the tutorial
%
% help model/sspace
% help model/get
% help model/specget
% help ftitle


